/*=========================================================================

  Program:   Visualization Toolkit
  Module:    vtkFiltersStatisticsGnuRObjectFactory.cxx

  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
  All rights reserved.
  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/

#include "vtkFiltersStatisticsGnuRObjectFactory.h"
#include "vtkVersion.h"

// Include all of the classes we want to create overrides for.

#include "vtkContingencyStatisticsGnuR.h"
#include "vtkCorrelativeStatisticsGnuR.h"
#include "vtkDescriptiveStatisticsGnuR.h"
#include "vtkPCAStatisticsGnuR.h"

vtkStandardNewMacro(vtkFiltersStatisticsGnuRObjectFactory)

// Now create the functions to create overrides with.

VTK_CREATE_CREATE_FUNCTION(vtkContingencyStatisticsGnuR)
VTK_CREATE_CREATE_FUNCTION(vtkCorrelativeStatisticsGnuR)
VTK_CREATE_CREATE_FUNCTION(vtkDescriptiveStatisticsGnuR)
VTK_CREATE_CREATE_FUNCTION(vtkPCAStatisticsGnuR)

vtkFiltersStatisticsGnuRObjectFactory::vtkFiltersStatisticsGnuRObjectFactory()
{

  this->RegisterOverride("vtkContingencyStatistics",
                         "vtkContingencyStatisticsGnuR",
                         "Override for vtkFiltersStatisticsGnuR module", 1,
                         vtkObjectFactoryCreatevtkContingencyStatisticsGnuR);
  this->RegisterOverride("vtkCorrelativeStatistics",
                         "vtkCorrelativeStatisticsGnuR",
                         "Override for vtkFiltersStatisticsGnuR module", 1,
                         vtkObjectFactoryCreatevtkCorrelativeStatisticsGnuR);
  this->RegisterOverride("vtkDescriptiveStatistics",
                         "vtkDescriptiveStatisticsGnuR",
                         "Override for vtkFiltersStatisticsGnuR module", 1,
                         vtkObjectFactoryCreatevtkDescriptiveStatisticsGnuR);
  this->RegisterOverride("vtkPCAStatistics",
                         "vtkPCAStatisticsGnuR",
                         "Override for vtkFiltersStatisticsGnuR module", 1,
                         vtkObjectFactoryCreatevtkPCAStatisticsGnuR);
}

const char * vtkFiltersStatisticsGnuRObjectFactory::GetVTKSourceVersion()
{
  return VTK_SOURCE_VERSION;
}

void vtkFiltersStatisticsGnuRObjectFactory::PrintSelf(ostream &os, vtkIndent indent)
{
  this->Superclass::PrintSelf(os, indent);
}

// Registration of object factories.
static unsigned int vtkFiltersStatisticsGnuRCount;

VTKFILTERSSTATISTICSGNUR_EXPORT void vtkFiltersStatisticsGnuR_AutoInit_Construct()
{
  if(++vtkFiltersStatisticsGnuRCount == 1)
    {
    vtkFiltersStatisticsGnuRObjectFactory* factory = vtkFiltersStatisticsGnuRObjectFactory::New();
    if (factory)
      {
      // vtkObjectFactory keeps a reference to the "factory",
      vtkObjectFactory::RegisterFactory(factory);
      factory->Delete();
      }
    }
}

VTKFILTERSSTATISTICSGNUR_EXPORT void vtkFiltersStatisticsGnuR_AutoInit_Destruct()
{
  if(--vtkFiltersStatisticsGnuRCount == 0)
    {
    // Do not call vtkObjectFactory::UnRegisterFactory because
    // vtkObjectFactory.cxx statically unregisters all factories.
    }
}
